# digits_autoenc.py
# condense the UCI digits to two dimensions
# CNTK 2.4 Anaconda3 4.1.1 (Python 3.5.2)

import numpy as np
import cntk as C
import matplotlib.pyplot as plt

# ===================================================================

def main():
  # 0. get started
  print("\nBegin UCI digits autoencoder with CNTK demo  \n")
  np.random.seed(1)

  # 1. load data into memory
  print("Loading 64-dimensional data into memory ")
  data_file = ".\\Data\\digits_uci_test_1797.txt"
  data_x = np.loadtxt(data_file, delimiter=",",
    usecols=range(0,64), dtype=np.float32)
  labels = np.loadtxt(data_file, delimiter=",",
    usecols=[64], dtype=np.float32)
  data_x = data_x / 16 

  # 2. define autoencoder
  print("Creating a 64-32-2-32-64 autoencoder ")
  my_init = C.initializer.glorot_uniform(seed=1)
  X = C.ops.input_variable(64, np.float32)  # inputs
  layer1 = C.layers.Dense(32, init=my_init,
    activation=C.ops.sigmoid)(X)
  layer2 = C.layers.Dense(2, init=my_init,
    activation=C.ops.sigmoid)(layer1)
  layer3 = C.layers.Dense(32, init=my_init,
    activation=C.ops.sigmoid)(layer2)
  layer4 = C.layers.Dense(64, init=my_init,
    activation=C.ops.sigmoid)(layer3)

  enc_dec = C.ops.alias(layer4)
  encoder = C.ops.alias(layer2)

  # 3. train model
  Y = C.ops.input_variable(64, np.float32)  # targets
  loss = C.squared_error(enc_dec, Y)
  learner = C.adam(enc_dec.parameters, lr=0.005, momentum=0.90)
  trainer = C.Trainer(enc_dec, (loss), [learner])
  N = len(data_x)
  bat_size = 2
  max_epochs = 10000
  print("\nStarting training")
  for i in range(0, max_epochs):
    rows = np.random.choice(N, bat_size, replace=False)
    trainer.train_minibatch({X:data_x[rows], Y:data_x[rows]})
    if i > 0 and i % int(max_epochs/10) == 0:
      mse = trainer.previous_minibatch_loss_average
      print("epoch = " + str(i) + " MSE = %0.4f " % mse)
  print("Training complete \n")  

  # 4. generate (x,y) pairs for each data item
  reduced = encoder.eval(data_x)

  # 5. graph the data in 2D
  print("Displaying 64-dim data in 2D: \n")
  plt.scatter(x=reduced[:, 0], y=reduced[:, 1],
    c=labels, edgecolors='none', alpha=0.9,
    cmap=plt.cm.get_cmap('nipy_spectral', 10), s=20)

  plt.xlabel('component 1')
  plt.ylabel('component 2')
  plt.colorbar()
  print("End autoencoder using CNTK demo ")
  plt.show()

if __name__ == "__main__":
  main()

